import json
from typing import List

from sidecar.const import Const


class KubApiPodReader:
    @staticmethod
    def get_pod_name(pod: {}) -> str:
        return pod['metadata']['name']

    @staticmethod
    def is_pod_ended(pod):
        return KubApiPodReader.get_pod_phase(pod) in ["Failed", "Succeeded"]

    @staticmethod
    def is_pod_terminating(pod: {}) -> bool:
        return True if pod['metadata'].get('deletionTimestamp', None) else False

    @staticmethod
    def get_pod_annotations(pod: {}) -> {}:
        return pod['metadata']['annotations']

    @staticmethod
    def get_apps_info_json(pod: {}) -> {}:
        return json.loads(KubApiPodReader.get_pod_annotations(pod)[Const.APPS])

    @staticmethod
    def safely_get_pod_ip(pod: {}) -> str:
        # pod status may not contain podIP key because by default the k8s api doesn't return uninitialized properties
        # and this property can be empty in some pod states
        return pod['status'].get('podIP', None)

    @staticmethod
    def get_pod_phase(pod: {}) -> str:
        return pod['status']['phase']

    @staticmethod
    def safely_get_container_id_for_app(app_name: str, pod: {}) -> str:
        container_statuses = pod['status'].get('containerStatuses', [])
        return next(iter(KubApiPodReader._get_container_id_from_status(container_status)
                         for container_status in container_statuses if container_status['name'] == app_name),
                    None)

    @staticmethod
    def safely_get_container_ids_in_pod(pod: {}) -> List[str]:
        container_ids = []
        container_statuses = pod['status'].get('containerStatuses', [])
        for container_status in container_statuses:
            container_id = KubApiPodReader._get_container_id_from_status(container_status)
            if container_id:
                container_ids.append(container_id)
        return container_ids

    @staticmethod
    def _get_container_id_from_status(container_status: {}) -> str:
        container_id = container_status.get('containerID', None)
        if container_id:
            # when container has restarted but hasn't start running yet ("waiting", e.g. due to CrashLoopBackOff),
            # containerID holds the id of the previous container for some reason
            # so we don't return it because that container has already terminated
            container_state = container_status['state']
            if container_state.get('waiting', None) is not None:
                container_id = None
        return container_id



import json
import threading
from abc import ABCMeta, abstractmethod
from logging import Logger

from sidecar.aws_status_maintainer import AWSStatusMaintainer
from sidecar.azure_clp.azure_status_maintainer import AzureStatusMaintainer
from sidecar.const import Const
from sidecar.kub_api_pod_reader import KubApiPodReader
from sidecar.kub_api_service import IKubApiService
from sidecar.services.service_status_state import ServiceStatusState
from sidecar.utils import Utils, CallsLogger


class IServiceStatusUpdater(metaclass=ABCMeta):
    def __init__(self, service_status_state: ServiceStatusState, logger: Logger):
        self.service_status_state = service_status_state
        self.logger = logger
        self.lock = threading.RLock()

    @CallsLogger.wrap
    def update(self, name: str, status: str):
        self.update_impl(name=name, status=status)
        self.service_status_state.update_status(name=name, status=status)

    @abstractmethod
    def update_impl(self, name: str, status: str):
        raise NotImplemented()


class K8SServiceStatusUpdater(IServiceStatusUpdater):

    def __init__(self, kub_api_service: IKubApiService, service_status_state: ServiceStatusState, logger: Logger):
        super().__init__(service_status_state, logger)
        self._k8s_service = kub_api_service

    @CallsLogger.wrap
    def update_impl(self, name: str, status: str):
        with self.lock:
            pod = Utils.retry_on_exception(
                func=lambda: self._k8s_service.get_pod_by_name(name=Const.SERVICE_EXECUTION_POD),
                logger=self.logger,
                logger_msg="trying to get service execution pod")

            if pod and not KubApiPodReader.is_pod_ended(pod) and not KubApiPodReader.is_pod_terminating(pod):
                service_data = json.loads(pod['metadata']['annotations'][name])
                service_data["status"] = status
                annotation_change = {name: json.dumps(service_data)}
                self._k8s_service.update_pod(Const.SERVICE_EXECUTION_POD, annotation_change)


class AwsServiceStatusUpdater(IServiceStatusUpdater):

    def __init__(self, status_maintainer: AWSStatusMaintainer, service_status_state: ServiceStatusState,
                 logger: Logger):
        super().__init__(service_status_state, logger)
        self._status_maintainer = status_maintainer

    @CallsLogger.wrap
    def update_impl(self, name: str, status: str):
        with self.lock:
            self._status_maintainer.update_service_status(name=name, status=status)


class AzureServiceStatusUpdater(IServiceStatusUpdater):

    def __init__(self, status_maintainer: AzureStatusMaintainer, service_status_state: ServiceStatusState,
                 logger: Logger):
        super().__init__(service_status_state, logger)
        self._status_maintainer = status_maintainer

    @CallsLogger.wrap
    def update_impl(self, name: str, status: str):
        with self.lock:
            self._status_maintainer.update_service_status(name=name, status=status)



import json
import os

from setuptools import setup, find_packages

ROOT = os.path.dirname(__file__)


def get_build_info_path():
    return os.path.join(ROOT, "build_info.json")


BUILD_INFO_PATH = get_build_info_path()


def get_build_info_json():
    with open(BUILD_INFO_PATH) as build_info_file:
        build_info_str = build_info_file.read().strip()
        build_info__json = json.loads(build_info_str)
        return build_info__json


BUILD_INFO_JSON = get_build_info_json()


def get_version():
    version = BUILD_INFO_JSON["version"]
    return version


def get_long_description():
    version = BUILD_INFO_JSON['version']
    # build_url = BUILD_INFO_JSON['build_url']
    branch = BUILD_INFO_JSON['branch']
    changeset = BUILD_INFO_JSON['changeset']
    return open('README.txt').read().format(VERSION=version, BRANCH=branch,
                                            CHANGESET=changeset)  # , BUILD_URL=build_url)


with open('requirements.txt') as f_required:
    requires = f_required.read().splitlines()

setup(
    name="cs18-sidecar",
    author="Quali",
    author_email="support@qualisystems.com",
    description='Sidecar - The CS18 instance',
    long_description=get_long_description(),
    packages=find_packages(exclude=['tests']),
    package_data={'': ['_init_.py', 'logzio.conf']},
    test_suite='nose.collector',
    install_requires=requires,
    version=get_version(),
    include_package_data=True,
    exclude_package_data={'': ['tests']},
    entry_points={
      'console_scripts': ['sidecar=sidecar.sidecar_api:run'],
    },
    classifiers=['Programming Language :: Python :: 3']
)
